//設定用
const Config = {
  //画面の解像度
  Screen: {
    Width: 256,//幅
    Height: 256,//高さ
    BackGroundColor: 0x000000,//背景色
  },
}
//読みこむデータリスト(名前：ファイルパス)
const Resource = {
  Title: "assets/title-scene.png",
  Game: "assets/game-scene.png"
}

//上のリストを読み込みやすい形に変えてます
const assets = [];
for (let key in Resource) {
  assets.push(Resource[key]);
}

let core;//ゲームの基幹プログラム用の変数

//ブラウザの読み込みが完了したら実行されます
window.onload = () => {
  //設定した画面サイズでcoreを作成
  core = new Game(Config.Screen.Width, Config.Screen.Height, Config.Screen.BackGroundColor);
  
  //データのロード
  core.preload(assets);

  //読み込み完了でメインシーンに切り替わります
  core.onload = () => {
    core.replaceScene(new TitleScene());
  }
}

//タイトルシーン
class TitleScene extends Container {
  constructor(){
    super();
    this.interactive = true;//タッチできるようにする

    //背景絵(取りあえず)
    const img = new EnchantSprite(Config.Screen.Width, Config.Screen.Height);
    img.image = core.resources[Resource.Title].texture;
    this.addChild(img)

    const style = new PIXI.TextStyle({
      fontFamily: 'sans-serif',
      fontSize: 40,
      fill: 0xffffff,
      fontWeight: 'bold',
      stroke: 0xFF0000,//縁取りの色
      strokeThickness: 4,//縁取りの太さ
    });
    const p = new PIXI.Text("おばけ", style);
    p.anchor.set(0.5, 0.5);
    p.position.set(Config.Screen.Width * 0.5, Config.Screen.Height * 0.15);
    this.addChild(p);

    this.isFading = true; 

    new FadeIn(this, 0x000000, 40, () => {
      this.isFading = false;
    });

    this.on("pointerup", (e) => {
      if(this.isFading) return;
      this.isFading = true; 

      const pos = e.data.getLocalPosition(e.currentTarget);
      new HoleOut(this, pos.x, pos.y, () => {
        core.replaceScene(new MainScene());
      });
    });
  }
  update(delta){
    super.update(delta);   
  }
}


//メインシーン
class MainScene extends Container {
  constructor(){
    super();
    this.interactive = true;//タッチできるようにする

    //背景絵
    const img = new EnchantSprite(Config.Screen.Width, Config.Screen.Height);
    img.image = core.resources[Resource.Game].texture;
    this.addChild(img)

    this.isFading = true; 

    new FadeIn(this, 0x000000, 40, () => {
      this.isFading = false;
    });

    this.on("pointerup", (e) => {
      if(this.isFading) return;
      this.isFading = true; 
      
      const pos = e.data.getLocalPosition(e.currentTarget);
      new HoleOut(this, pos.x, pos.y, () => {
        core.replaceScene(new TitleScene());
      });
    });
  }
  update(delta){
    super.update(delta);   
  }
}

//穴が閉じるやつ
class HoleOut extends Graphics {
  constructor(container, x, y, callback){
    super();
    container.addChild(this);
    this.color = 0x000000;
    this.center_x = x;
    this.center_y = y;
    let w, h;
    //半径を出すために必要な幅と高さを出す
    if(x > Config.Screen.Width * 0.5){
      w = x;
    }else{
      w = Config.Screen.Width - x;
    }
    if(y > Config.Screen.Height * 0.5){
      h = y;
    }else{
      h = Config.Screen.Height - y;
    }

    //円の半径(一番長い対角線を半径にする)
    this.radius = Math.sqrt(w * w + h * h);
    
    this.onFinished = () => {};
    if(callback){
      this.onFinished = callback;
    }
  } 
  update(delta){
    super.update(delta);
    this.clear();

    //可能性のある最大範囲を黒く塗っている(穴より小さいとバグる)
    this.beginFill(0x000000);
    this.drawRect(-Config.Screen.Width*1.5, -Config.Screen.Height*1.5, Config.Screen.Width*4, Config.Screen.Height*4);
    this.endFill();

    //穴を開ける
    this.beginHole();
    this.drawCircle(this.center_x, this.center_y, this.radius);
    this.endHole();

    //
    let s = this.radius * 0.02;
    if(s < 1) s = 1;//小さくなりすぎると終わらないので
    this.radius -= s;
    if(this.radius <= 0){
      this.onFinished();
      this.destroy();
      return;
    }   
  }
}

class FadeIn extends Graphics {
  constructor(container, color, frameLength, callback){
    super();
    container.addChild(this);
    this.rectFill(0, 0, Config.Screen.Width, Config.Screen.Height, color);
    this.step = 1 / frameLength;
    this.onFinished = () => {};
    if(callback){
      this.onFinished = callback;
    }
  } 
  update(delta){
    super.update(delta);
    this.alpha -= this.step;
    if(this.alpha <= 0){
      this.onFinished();
      this.destroy();
      return;
    }   
  }
}
